if not oUF then return end

--[[------------------------------------------------------------------------------------------------------
oUF_VillainAuras by Munglunch (originally by Astromech)

Example use:

local function fMungAuras(self, unit)
	local barspellz = CreateFrame("Frame", nil, self)
	barspellz:SetPoint("TOPLEFT", self.panel, 2, -2)
	barspellz:SetPoint("BOTTOMRIGHT", self.panel, -2, 2)
	barspellz:SetFrameLevel(120)
	barspellz.presentAlpha = 1
	barspellz.missingAlpha = 0
	barspellz.bars = {}

	local buffs = SVMod.barSpells
	
	for name, spell in pairs(buffs) do
		local f = CreateFrame("Frame",nil,barspellz)
		f:SetWidth(SVMod.Scale(60))
		f:SetHeight(SVMod.Scale(10))
		f:SetPoint("CENTER",0,0)
		f.spellID = spell
		f.anyUnit = false
		
		local bar = CreateFrame("StatusBar", _, f)
		bar:SetStatusBarTexture(SVMedia.blank)
		bar:SetStatusBarColor(181/255, 13/255, 13/255, 0.5)
		bar:SetMinMaxValues(0, 1)
		bar:SetAllPoints(f)
		f.bar = bar

		barspellz.bars[spell] = f
	end
	self.VillainAuras = barspellz
end
-----------------------------------------------------------------------------------------------------------]]

local _, ns = ...
local oUF = ns.oUF or _G.oUF
assert(oUF, "oUF_VillainAuras cannot find an instance of oUF. If your oUF is embedded into a layout, it may not be embedded properly.")

local UnitBuff, UnitDebuff, UnitGUID = UnitBuff, UnitDebuff, UnitGUID
local GUIDs = {}

local PLAYER_UNITS = {
	player = true,
	vehicle = true,
	pet = true,
}

local setupGUID
do 
	local cache = setmetatable({}, {__type = "k"})

	local frame = CreateFrame"Frame"
	frame:SetScript("OnEvent", function(self, event)
		for k,t in pairs(GUIDs) do
			GUIDs[k] = nil
			for a in pairs(t) do
				t[a] = nil
			end
			cache[t] = true
		end
	end)
	frame:RegisterEvent"PLAYER_REGEN_ENABLED"
	frame:RegisterEvent"PLAYER_ENTERING_WORLD"

	function setupGUID(guid)
		local t = next(cache)
		if t then
			cache[t] = nil
		else
			t = {}
		end
		GUIDs[guid] = t
	end
end


local function resetbar(bar, frame, duration, remaining)
	if bar.onlyShowMissing then
		bar:Hide()
	else
		bar:Show()
		bar:SetAlpha(frame.presentAlpha)
	end
end

local function expirebar(bar, frame)
	if bar.onlyShowPresent then
		bar:Hide()
	else
		bar:SetAlpha(frame.missingAlpha)
		bar:Show()
	end
end

local function UpdateBar(self)
	local duration = self.Duration
	local timeLeft = self.EndTime-GetTime()
	local roundedt = math.floor(timeLeft*10.5)/10
	self.bar:SetValue(timeLeft/duration)
end

local found = {}
local function Update(frame, event, unit)
	if frame.unit ~= unit then return end
	local watch = frame.VillainAuras
	local index, bars = 1, watch.watched
	local _, name, _, _, duration, remaining, caster, key, bar, spellid 
	local filter = "HELPFUL"
	local guid = UnitGUID(unit)
	if not GUIDs[guid] then setupGUID(guid) end

	for key, bar in pairs(bars) do
		bar:Hide()
	end
	
	while true do
		name, _, _, _, _, duration, remaining, caster, _, _, spellid = UnitAura(unit, index, filter)
		if not name then 
			if filter == "HELPFUL" then
				filter = "HARMFUL"
				index = 1
			else
				break
			end
		else
			if watch.strictMatching then
				key = spellID
			else
				key = name
			end
			bar = bars[key]
			if bar and (bar.anyUnit or (caster and bar.fromUnits[caster])) then
				bar.EndTime = remaining
				bar.Duration = duration
				bar:SetScript("OnUpdate", UpdateBar)
				resetbar(bar, watch, duration, remaining)
				GUIDs[guid][key] = true
				found[key] = true
			end
			index = index + 1
		end
	end

	for key in pairs(GUIDs[guid]) do
		if bars[key] and not found[key] then
			expirebar(bars[key], watch)
		end
	end

	for k in pairs(found) do
		found[k] = nil
	end
end

local function setupbars(self)

	local watch = self.VillainAuras
	local bars = watch.bars
	watch.watched = {}
	if not watch.missingAlpha then watch.missingAlpha = 0.75 end
	if not watch.presentAlpha then watch.presentAlpha = 1 end

	for _,bar in pairs(bars) do

		local name, _, _ = GetSpellInfo(bar.spellID)
		if not name then error("oUF_VillainAuras error: no spell with "..tostring(bar.spellID).." spell ID exists") end
		bar.name = name

		if bar.onlyShowMissing == nil then
			bar.onlyShowMissing = watch.onlyShowMissing
		end
		if bar.onlyShowPresent == nil then
			bar.onlyShowPresent = watch.onlyShowPresent
		end
		if bar.fromUnits == nil then
			bar.fromUnits = watch.fromUnits or PLAYER_UNITS
		end
		if bar.anyUnit == nil then
			bar.anyUnit = watch.anyUnit
		end

		if watch.strictMatching then
			watch.watched[bar.spellID] = bar
		else
			watch.watched[name] = bar
		end
		
		if watch.PostCreateBar then watch:PostCreateBar(bar, bar.spellID, name, self) end
	end
end

local function Enable(self)
	if self.VillainAuras then
		self:RegisterEvent("UNIT_AURA", Update)
		setupbars(self)
		return true
	else
		return false
	end
end

local function Disable(self)
	if self.VillainAuras then
		self:UnregisterEvent("UNIT_AURA", Update)
		for _,bar in pairs(self.VillainAuras.bars) do
			bar:Hide()
		end
	end
end

oUF:AddElement("VillainAuras", Update, Enable, Disable)