--[[
  /$$$$$$                                          /$$    /$$ /$$ /$$ /$$           /$$          
 /$$__  $$                                        | $$   | $$|__/| $$| $$          |__/          
| $$  \__/ /$$   /$$  /$$$$$$   /$$$$$$   /$$$$$$ | $$   | $$ /$$| $$| $$  /$$$$$$  /$$ /$$$$$$$ 
|  $$$$$$ | $$  | $$ /$$__  $$ /$$__  $$ /$$__  $$|  $$ / $$/| $$| $$| $$ |____  $$| $$| $$__  $$
 \____  $$| $$  | $$| $$  \ $$| $$$$$$$$| $$  \__/ \  $$ $$/ | $$| $$| $$  /$$$$$$$| $$| $$  \ $$
 /$$  \ $$| $$  | $$| $$  | $$| $$_____/| $$        \  $$$/  | $$| $$| $$ /$$__  $$| $$| $$  | $$
|  $$$$$$/|  $$$$$$/| $$$$$$$/|  $$$$$$$| $$         \  $/   | $$| $$| $$|  $$$$$$$| $$| $$  | $$
 \______/  \______/ | $$____/  \_______/|__/          \_/    |__/|__/|__/ \_______/|__/|__/  |__/
                    | $$                                                                         
                    | $$                                                                         
                    |__/                                                                         
    ________                   __  ____        __  __              ______            __                
   / ____/ /__  ______  __  __/ /_/ __ )__  __/ /_/ /_____  ____  / ____/__  _______/ /_____  ____ ___ 
  / /_  / // / / / __ \/ / / / __/ __  / / / / __/ __/ __ \/ __ \/ /    / / / / ___/ __/ __ \/ __ `__ \
 / __/ / // /_/ / /_/ / /_/ / /_/ /_/ / /_/ / /_/ /_/ /_/ / / / / /___ / /_/ (__  ) /_/ /_/ / / / / / /
/_/   /_/ \__, /\____/\__,_/\__/_____/\__,_/\__/\__/\____/_/ /_/\____/ \__,_/____/\__/\____/_/ /_/ /_/ 
         /____/                                                                                        

Originally created by: Another .Re-engineered for SVUI by: Munglunch
 ]]--

--const
local BUTTON_PLACE_SIZE = 36
local BUTTON_PLACE_OFFSET = 4
local MIN_VISIBLE_BUTTONS_SETTINGS_MODE = 4
local UPDATE_TIME_PER_EVENT = 0.25
local ACCEPTABLE_COMMANDS = {
	["spell"] = true,
	["item"] = true,
	["macro"] = true,
	["companion"] = true
}

--arrow button directions
local DIR_UP	= 1
local DIR_LEFT	= 2
local DIR_DOWN	= 3
local DIR_RIGHT	= 4
local ARROW_BUTTON_DEF_WIDTH = 36
local ARROW_BUTTON_DEF_HEIGHT, ARROW_BUTTON_MIN_HEIGHT, ARROW_BUTTON_MAX_HEIGHT = 18, 5, 36
local ARROW_BUTTON_DEF_ALPHA_MULTIPLIER = 1
local BF_SETTINGS_GROUP = "Button Facade"


--variables
local AddonName, AddonTable = ...
local ButtonFrames = {}
local ArrowButtons = {}
local EventHandlersTable = {}
local EnteringWorldOrVariablesLoaded = false
local SettingsMode
local ArrowButtonsHeight
local ShowBorders
local AlphaMultiplier
local EnableMouseover
local StoredCursor = {}
local StubFrame = CreateFrame("Frame", nil, UIParent, "SecureHandlerBaseTemplate")
--Button Facade stuff
local LBF, LBFMasterGroup
local BF_Table = {}
			


--[[structures
ActionButton
	- customFlyout
ArrowBtn
	- parent (ActionButton)
	- FlyoutArrow		
	- expandDir: up, left, down, right
	- expanded
	- FlyoutListTable = {
		- [index] = {
			- ["command"] = command, 
			- ["value" = value, 
			- ["subValue"] = subValue
			- },
		- ...
		- }

	- FlyoutListFrame	--also through SetFrameRef
		- ButtonList
]]

local ArrowButtonOnClickSnippet = [=[
	local B_PLACE_SIZE = 36
	local B_PLACE_OFFSET = 4

	local D_UP		= 1
	local D_LEFT	= 2
	local D_DOWN	= 3
	local D_RIGHT	= 4

	if button == "LeftButton" then
		lfRef = self:GetFrameRef("FlyoutListFrame_Ref")
		
		local width = B_PLACE_SIZE + 2 * B_PLACE_OFFSET
		local height = self:GetAttribute("bcount") * (B_PLACE_SIZE + B_PLACE_OFFSET) + B_PLACE_OFFSET
		
		if self:GetAttribute("expandDir") == D_UP or self:GetAttribute("expandDir") == D_DOWN then
			lfRef:SetWidth(width)
			lfRef:SetHeight(height)
		else
			lfRef:SetWidth(height)
			lfRef:SetHeight(width)
		end

		if self:GetAttribute("expanded") then
			self:SetAttribute("expanded", false)
			lfRef:Hide()
		else
			--close other opened lists
			local stub = self:GetFrameRef("StubFrame")
			if stub and stub:GetAttribute("unique-list") then
				local count = stub:GetAttribute("ArrowButtonsCount")
				if count then
					for i = 1, count do
						local ab = stub:GetFrameRef("ArrowButtons"..i)
						if ab and ab:GetAttribute("expanded") then
							local lf = ab:GetFrameRef("FlyoutListFrame_Ref")
							if lf then
								ab:SetAttribute("expanded", false)
								lf:Hide()
							end
						end
					end
				end
			end
			
			self:SetAttribute("expanded", true)
			lfRef:Show()
		end
	end
]=]

local ArrowButtonOnEnterSnippet = [=[
	local B_PLACE_SIZE = 36
	local B_PLACE_OFFSET = 4

	local D_UP		= 1
	local D_LEFT	= 2
	local D_DOWN	= 3
	local D_RIGHT	= 4

	lfRef = self:GetFrameRef("FlyoutListFrame_Ref")
	
	local width = B_PLACE_SIZE + 2 * B_PLACE_OFFSET
	local height = self:GetAttribute("bcount") * (B_PLACE_SIZE + B_PLACE_OFFSET) + B_PLACE_OFFSET
	
	if self:GetAttribute("expandDir") == D_UP or self:GetAttribute("expandDir") == D_DOWN then
		lfRef:SetWidth(width)
		lfRef:SetHeight(height)
	else
		lfRef:SetWidth(height)
		lfRef:SetHeight(width)
	end

	if self:GetAttribute("expanded") then
		self:SetAttribute("expanded", false)
		lfRef:Hide()
	else
		--close other opened lists
		local stub = self:GetFrameRef("StubFrame")
		if stub and stub:GetAttribute("unique-list") then
			local count = stub:GetAttribute("ArrowButtonsCount")
			if count then
				for i = 1, count do
					local ab = stub:GetFrameRef("ArrowButtons"..i)
					if ab and ab:GetAttribute("expanded") then
						local lf = ab:GetFrameRef("FlyoutListFrame_Ref")
						if lf then
							ab:SetAttribute("expanded", false)
							lf:Hide()
						end
					end
				end
			end
		end
		
		self:SetAttribute("expanded", true)
		lfRef:Show()
	end
]=]

local ArrowButtonCombatSnippet = [=[
	if tonumber(newstate) == 1 and self:GetAttribute("bcount") == 0 and self:IsVisible() then
		self:SetAttribute("expanded", false)
		self:Hide()
	end
]=]

local ArrowButtonVehicleSnippet = [=[
	if tonumber(newstate) == 1 and self:IsVisible() then
		self:SetAttribute("was_visible_vehicle", true)
		self:SetAttribute("expanded", false)
		self:Hide()
	elseif self:GetAttribute("was_visible_vehicle") then
		self:SetAttribute("was_visible_vehicle", false)
		self:SetAttribute("expanded", true)
		self:Show()
	end
]=]

local FlyoutListFrameCombatSnippet = [=[
	if tonumber(newstate) == 1 and self:GetAttribute("bcount") == 0 and self:IsVisible() then
		self:Hide()
	end
]=]

local FlyoutListFrameVehicleSnippet = [=[
	if tonumber(newstate) == 1 and self:IsVisible() then
		self:SetAttribute("was_visible_vehicle", true)
		self:Hide()
	elseif self:GetAttribute("was_visible_vehicle") then
		self:SetAttribute("was_visible_vehicle", false)
		self:Show()
	end
]=]

local function RegisterFrames()
	ButtonFrames = {}
	local frame = EnumerateFrames()
	while frame do
		if frame.IsProtected and frame.GetObjectType and frame.GetScript and frame:GetObjectType() == "CheckButton" and frame:IsProtected() then
			local script = frame:GetScript("OnClick")
			if script == ActionButton1:GetScript("OnClick") or tostring(frame:GetName()):match("^BT4Button%d+$") then
				tinsert(ButtonFrames, frame)
			end
		end
		frame = EnumerateFrames(frame)
	end
end

local function GetSpellName(spell, ...)
	local name, rank = GetSpellBookItemName(spell, ...)

	if (not rank) then
		rank = ""
	end
	if (name) then
		return name.."("..rank..")", name
	end
	return nil, nil
end

local function FindSpellSlot(spellName)
	local slot = 1
	while true do
		local name = GetSpellName(slot, BOOKTYPE_SPELL)
		if (not name) then
			break
		end
		if (spellName == name) then
			return slot
		end
		slot = slot + 1
	end
	return nil
end

local function SetArrowTextureRotation(FlyoutArrow, expandDir, expanded)
	if expanded then
		if expandDir == DIR_UP then
			SetClampedTextureRotation(FlyoutArrow, 180)
		elseif expandDir == DIR_LEFT then
			SetClampedTextureRotation(FlyoutArrow, 90)
		elseif expandDir == DIR_DOWN then
			SetClampedTextureRotation(FlyoutArrow, 0)
		else --if expandDir == DIR_RIGHT then
			SetClampedTextureRotation(FlyoutArrow, 270)
		end
	else
		if expandDir == DIR_UP then
			SetClampedTextureRotation(FlyoutArrow, 0)
		elseif expandDir == DIR_LEFT then
			SetClampedTextureRotation(FlyoutArrow, 270)
		elseif expandDir == DIR_DOWN then
			SetClampedTextureRotation(FlyoutArrow, 180)
		else --if expandDir == DIR_RIGHT then
			SetClampedTextureRotation(FlyoutArrow, 90)
		end
	end
end

local function SetArrowFrameAnchor(arrowBtn, frame, expandDir)
	arrowBtn:ClearAllPoints()
	if expandDir == DIR_UP then
		arrowBtn:SetPoint("TOP", frame, "TOP", 0, 4)
	elseif expandDir == DIR_LEFT then
		arrowBtn:SetPoint("LEFT", frame, "LEFT", -4, 0)
	elseif expandDir == DIR_DOWN then
		arrowBtn:SetPoint("BOTTOM", frame, "BOTTOM", 0, -4)
	else
		arrowBtn:SetPoint("RIGHT", frame, "RIGHT", 4, 0)
	end
end

local function SetArrowFrameSize(arrowBtn, expandDir)
	if expandDir == DIR_UP or expandDir == DIR_DOWN then
		arrowBtn:SetWidth(ARROW_BUTTON_DEF_WIDTH)
		arrowBtn:SetHeight(ArrowButtonsHeight)
	else
		arrowBtn:SetWidth(ArrowButtonsHeight)
		arrowBtn:SetHeight(ARROW_BUTTON_DEF_WIDTH)
	end
end

local function AttachArrow(frame)
	if frame and not(InCombatLockdown()) then --and frame:IsVisible()
		local actionType = GetActionInfo(frame.action)
		if (actionType == "flyout") then
			return
		end
		
		local actnBtnName = frame:GetName()
		local currentSet = GetActiveTalentGroup()
		local expandDir = DIR_UP
		
		if not(ArrowButtons[actnBtnName]) then --create new
			frame.customFlyout = true
			local tempFrame = _G[actnBtnName.."ArrowBtn"]
			if tempFrame then
				ArrowButtons[actnBtnName] = tempFrame --reuse frame
			else
				ArrowButtons[actnBtnName] = CreateFrame("CheckButton", actnBtnName.."ArrowBtn", frame, "CustomFlyoutArrowButtonTemplate")
				--set reference in stub
				local count = StubFrame:GetAttribute("ArrowButtonsCount")
				if not(count) then
					count = 1
				else
					count = count + 1
				end
				StubFrame:SetAttribute("ArrowButtonsCount", count)
				StubFrame:SetFrameRef("ArrowButtons"..count, ArrowButtons[actnBtnName])
				ArrowButtons[actnBtnName]:SetFrameRef("StubFrame", StubFrame)
			end
			arrowBtn = ArrowButtons[actnBtnName]
			--arrowBtn:SetFrameLevel(arrowBtn:GetParent():GetFrameLevel() + 1)

			arrowBtn:SetAttribute("_onclick", ArrowButtonOnClickSnippet)
			arrowBtn:SetAttribute("_onenter", ArrowButtonOnEnterSnippet)
			if not(EnableMouseover) then
				arrowBtn:SetScript("OnEnter", CustomFlyoutArrowButton_OnEnter)
			end
			arrowBtn:RegisterForClicks("LeftButtonUp", "RightButtonUp")

			--self, stateid, newstate
			arrowBtn:SetAttribute("_onstate-combat", ArrowButtonCombatSnippet)
			RegisterStateDriver(arrowBtn, "combat", "[nocombat] 0; [combat] 1;")

			arrowBtn:SetAttribute("_onstate-vehicle", ArrowButtonVehicleSnippet)
			RegisterStateDriver(arrowBtn, "vehicle", "[novehicleui] 0; [vehicleui] 1;")
			
			if (frame:GetParent() == MultiBarRight or frame:GetParent() == MultiBarLeft) then
				expandDir = DIR_LEFT
			end
			arrowBtn.parent = frame
		else
			arrowBtn = ArrowButtons[actnBtnName]
		end
		
		if SVUIFlyoutCustom_Settings[actnBtnName] and SVUIFlyoutCustom_Settings[actnBtnName][currentSet] then
			arrowBtn.FlyoutListTable = SVUIFlyoutCustom_Settings[actnBtnName][currentSet]
		else
			arrowBtn.FlyoutListTable = {}
		end
		
		if SVUIFlyoutCustom_Settings["expandDir"][currentSet][actnBtnName] then
			expandDir = SVUIFlyoutCustom_Settings["expandDir"][currentSet][actnBtnName]
		end
		SetArrowFrameAnchor(arrowBtn, frame, expandDir)
		arrowBtn:SetAttribute("expandDir", expandDir)
		
		SetArrowFrameSize(arrowBtn, expandDir)
		
		arrowBtn:SetAttribute("expanded", false)
		arrowBtn:SetAttribute("bcount", #arrowBtn.FlyoutListTable)
		SetArrowTextureRotation(arrowBtn.FlyoutArrow, expandDir, false)
		arrowBtn.FlyoutArrow:Show()
		if (#arrowBtn.FlyoutListTable > 0) then
			arrowBtn:Show()
			arrowBtn:SetAlpha(0.75 * AlphaMultiplier)
		else
			arrowBtn:Hide()
		end
		FlyoutButtonCustom_Update(arrowBtn)
		
		return true
	end
	return false
end

local function ShowArrow(frame)
	if frame and frame:IsVisible() and not(InCombatLockdown()) then
		local arrowBtn = ArrowButtons[frame:GetName()]
		if not(arrowBtn) then 
			return false
		end
		
		SetArrowTextureRotation(arrowBtn.FlyoutArrow, arrowBtn:GetAttribute("expandDir"), arrowBtn:GetAttribute("expanded"))
	
		arrowBtn:Show()
		arrowBtn.FlyoutArrow:Show()
		if (#arrowBtn.FlyoutListTable > 0) then
			arrowBtn:SetAlpha(0.75 * AlphaMultiplier)
		else
			arrowBtn:SetAlpha(0.5 * AlphaMultiplier)
		end
		return true
	end
	return false
end

local function HideArrow(frame)
	if frame and frame:IsVisible() and not(InCombatLockdown()) then
		local arrowBtn = ArrowButtons[frame:GetName()]
		if not(arrowBtn) then 
			return false
		end
		
		if (arrowBtn:GetAttribute("bcount") > 0) then
			arrowBtn:SetAlpha(0.75 * AlphaMultiplier)
		else
			arrowBtn:SetAttribute("expanded", false)
			arrowBtn.FlyoutListFrame:Hide()
			arrowBtn.FlyoutArrow:Hide()
			arrowBtn:Hide()
		end
		return true
	end
	return false
end

local function AttachToAllActionButtons()
	for _, frame in ipairs(ButtonFrames) do
		if frame.action or frame._state_action then
			AttachArrow(frame)
		end
	end
end

local function ShowAllFlyoutButtons()
	for _, frame in ipairs(ButtonFrames) do
		if frame.action or frame._state_action then
			ShowArrow(frame)
		end
	end
end

local function HideUnusedFlyoutButtons()
	for _, frame in ipairs(ButtonFrames) do
		if frame.action or frame._state_action then
			HideArrow(frame)
		end
	end
end

local function HideFlyoutFrames()
	for _, frame in ipairs(ButtonFrames) do
		if frame.action or frame._state_action then
			if frame and not(InCombatLockdown()) then
				local arrowBtn = ArrowButtons[frame:GetName()]
				if (arrowBtn) then 
					arrowBtn:SetAttribute("expanded", false)
					arrowBtn.FlyoutListFrame:Hide()
				end
			end
		end
	end
end



--[[ button update/set functions ]]

local function GetCompanionInfoCustom(compType, compName)
	local creatureName, creatureSpellID, spellName, icon
	for i = 1, GetNumCompanions(compType) do
		--creatureID, creatureName, creatureSpellID, icon, issummoned
		_, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo(compType, i)
		local spellName = GetSpellInfo(creatureSpellID)
		if spellName == compName then
			return i, creatureName, creatureSpellID, spellName, icon, issummoned
		end
	end
	
	return nil
end

function CustomFlyoutListButton_UpdateTooltip(self)
	if (GetCVar("UberTooltips") == "1") then
		GameTooltip_SetDefaultAnchor(GameTooltip, self)
	else
		local lf = self:GetParent() --list->arrowBtn->actBtn->Bar
		local arrBtn = lf:GetParent() --arrowBtn->actBtn->Bar
		local actBtn = arrBtn.parent --actBtn->Bar
		local bar = actBtn:GetParent()
		if (bar == MultiBarBottomRight or bar == MultiBarRight or bar == MultiBarLeft) then
			GameTooltip:SetOwner(self, "ANCHOR_LEFT")
		else
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		end
	end
	
	if self.tooltipType and self.tooltipValue then
		if self.tooltipType == "hyperlink" then
			GameTooltip:SetHyperlink(self.tooltipValue)
		elseif self.tooltipType == "spell" then
			GameTooltip:SetSpellBookItem(self.tooltipValue, BOOKTYPE_SPELL)
		elseif self.tooltipType == "item" then
			local _, hyperLink = GetItemInfo(self.tooltipValue)
			if (hyperLink) then
				GameTooltip:SetHyperlink(hyperLink)
			end
		elseif self.tooltipType == "macro" then
			GameTooltip:SetText(self.tooltipValue, 1.0, 1.0, 1.0)
		end
	end	
end

local function ListButton_SetTooltip(self, command, value, subValue)
	self.tooltipValue = nil
	self.tooltipType = nil
	
	if (command == "spell") then
		if (subValue == "MOUNT" or subValue == "CRITTER") then
			_, _, creatureSpellID = GetCompanionInfoCustom(subValue, value)
			self.tooltipType = "hyperlink"
			self.tooltipValue = "spell:"..creatureSpellID
		else
			self.tooltipType = command
			self.tooltipValue = FindSpellSlot(value)
		end
	elseif (command == "item") then
		self.tooltipType = command
		self.tooltipValue = value
	elseif (command == "macro") then
		self.tooltipType = command
		self.tooltipValue = value
	end
end

local function ListButton_UpdateTexture(self, command, value, subValue)
	local icon = _G[self:GetName().."Icon"]
	local texture = nil
	
	if (command == "spell") then
		if (subValue == "MOUNT" or subValue == "CRITTER") then
			_, _, _, _, texture = GetCompanionInfoCustom(subValue, value)
		else
			texture = GetSpellTexture(value)
		end
	elseif (command == "item") then
		texture = GetItemIcon(value)
	elseif (command == "macro") then
		_, texture = GetMacroInfo(value)
	end

	if (texture) then
		icon:SetTexture(texture)
		icon:SetVertexColor(1.0, 1.0, 1.0, 1.0)
		icon:Show()
		if ShowBorders then
			self:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2")
		else
			self:SetNormalTexture("") --remove border
		end
	elseif (command == "spell") or (command == "macro") then
		icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		icon:SetVertexColor(1.0, 1.0, 1.0, 0.5)
		icon:Show()
		self:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2")		
	else
		local buttonCooldown = _G[self:GetName().."Cooldown"]
		icon:Hide()
		buttonCooldown:Hide()
		self:SetNormalTexture("Interface\\Buttons\\UI-Quickslot")
	end
end

local function ListButton_UpdateChecked(button, command, value, subValue)
	local result = false
	if (command == "spell") then
		if (subValue == "MOUNT" or subValue == "CRITTER") then
			_, _, _, _, _, result = GetCompanionInfoCustom(subValue, value)
			local spellName = UnitCastingInfo("player")
			result = result or spellName == value
		else
			result = IsCurrentSpell(value) or IsAutoRepeatSpell(value)
		end
	elseif (command == "item") then
		result = IsCurrentItem(value)
	elseif (command == "macro") then
		--nothing, this isn't action bar
	end
	
	button:SetChecked(result)
end

local function ListButton_UpdateEquipped(button, command, value, subValue)
	local border = _G[button:GetName().."Border"]
	if (command == "item") then
		if IsEquippedItem(value) then
			border:SetVertexColor(0, 1.0, 0, 0.35)
			border:Show()
		else
			border:Hide()
		end
	elseif (command == "macro") then
		--todo?
	end
end

local function ListButton_UpdateCooldown(button, command, value, subValue)
	local cooldown = _G[button:GetName().."Cooldown"]
	if (command == "spell") then
		if (subValue == "MOUNT" or subValue == "CRITTER") then
			local index = GetCompanionInfoCustom(subValue, value)
			if (index) then
				CooldownFrame_SetTimer(cooldown, GetCompanionCooldown(subValue, index))
			end
		else
			local start, duration, enable = GetSpellCooldown(value)
			if start then
				CooldownFrame_SetTimer(cooldown, start, duration, enable)
			else
				CooldownFrame_SetTimer(cooldown, 0, 0, 0)
				cooldown:Hide()
			end
		end
	elseif (command == "item") then
		CooldownFrame_SetTimer(cooldown, GetItemCooldown(value))
	elseif (command == "macro") then
		local index = subValue --GetMacroIndexByName(value)
		if (index) then
			local name, rank = GetMacroSpell(index)
			if (name) then
				local spell = name.."("..rank..")"
				local start, duration, enable = GetSpellCooldown(spell)
				if start then
					CooldownFrame_SetTimer(cooldown, start, duration, enable)
				else
					CooldownFrame_SetTimer(cooldown, 0, 0, 0)
					cooldown:Hide()
				end
			end
		end
	end
end

local function ListButton_UpdateUsable(button, command, value, subValue)
	local isUsable, notEnoughMana
	if (command == "spell") then
		if (subValue == "MOUNT" or subValue == "CRITTER") then
			--nothing works in Darkbreak Cove for mounts, Blizz using IsUsableAction(action_bar_slot)
			isUsable = (subValue == "MOUNT" and IsOutdoors()) or subValue == "CRITTER"
		else
			isUsable, notEnoughMana = IsUsableSpell(value)
		end
	elseif (command == "item") then
		isUsable, notEnoughMana = IsUsableItem(value)
	elseif (command == "macro") then
		--this isn't action bar, always usable
		isUsable = true
	end
	
	local icon = _G[button:GetName().."Icon"]
	local ntexture = _G[button:GetName().."NormalTexture"]
	if (isUsable) then
		icon:SetVertexColor(1.0, 1.0, 1.0)
		ntexture:SetVertexColor(1.0, 1.0, 1.0)
	elseif (notEnoughMana and not(subValue == "MOUNT" or subValue == "CRITTER")) then
		icon:SetVertexColor(0.5, 0.5, 1.0)
		ntexture:SetVertexColor(0.5, 0.5, 1.0)
	else
		icon:SetVertexColor(0.4, 0.4, 0.4)
		ntexture:SetVertexColor(1.0, 1.0, 1.0)
	end	
end

local function ListButton_UpdateButtonText(self, command, value, subValue)
	local text = _G[self:GetName().."Count"]
	local actionName = _G[self:GetName().."Name"]
	
	text:SetText("")
	actionName:SetText("")
	
	if (command == "spell" and IsConsumableSpell(value)) then
		text:SetText(GetSpellCount(value))
	elseif (command == "item" and IsConsumableItem(value)) then
		text:SetText(GetItemCount(value))
	elseif (command == "item" and GetItemCount(value) > 1) then
		text:SetText(GetItemCount(value))
	elseif (self:GetAttribute("type") == "macro") then
		-- name, iconTexture, body, isLocal = GetMacroInfo("name" or macroSlot)
		actionName:SetText(GetMacroInfo(subValue)) --by slot
	end
	text:Show()
end

local function UpdateListButton(button, command, value, subValue)
	ListButton_SetTooltip(button, command, value, subValue)
	ListButton_UpdateTexture(button, command, value, subValue)
	ListButton_UpdateCooldown(button, command, value, subValue)
	ListButton_UpdateChecked(button, command, value, subValue)
	ListButton_UpdateEquipped(button, command, value, subValue)
	ListButton_UpdateUsable(button, command, value, subValue)
	ListButton_UpdateButtonText(button, command, value, subValue)
end

local function SetListButton(button, command, value, subValue)
	--print("SetListButton "..tostring(command)..", "..tostring(value)..", "..tostring(subValue))
	button:SetAttribute(command, value)
	button:SetAttribute("type", command)
	if command == "item" then
		local itemname = GetItemInfo(value)
		button:SetAttribute("item", itemname)
	--elseif command == "macro" then 
		--SetAttribute("macrotext", subValue) or SetAttribute("macro", macroindex)?
	end
	
	ListButton_SetTooltip(button, command, value, subValue)
	UpdateListButton(button, command, value, subValue)
	button:Show()
end



local function SetListButtonAnchor(btn, parent, idx, expandDir)
	btn:ClearAllPoints()
	if expandDir == DIR_UP then
		btn:SetPoint("BOTTOM", parent, "BOTTOM", 0, idx * BUTTON_PLACE_OFFSET + (idx - 1) * BUTTON_PLACE_SIZE)
	elseif expandDir == DIR_LEFT then
		btn:SetPoint("RIGHT", parent, "RIGHT", - (idx * BUTTON_PLACE_OFFSET + (idx - 1) * BUTTON_PLACE_SIZE), 0)
	elseif expandDir == DIR_DOWN then
		btn:SetPoint("TOP", parent, "TOP", 0, - (idx * BUTTON_PLACE_OFFSET + (idx - 1) * BUTTON_PLACE_SIZE))
	else
		btn:SetPoint("LEFT", parent, "LEFT", idx * BUTTON_PLACE_OFFSET + (idx - 1) * BUTTON_PLACE_SIZE, 0)
	end
end

local function AttachListButton(parent, idx, expandDir)
	local btn = parent.ButtonList[idx]
	if not(btn) then
		parent.ButtonList[idx] = CreateFrame("CheckButton", parent:GetName().."ListButton"..idx, parent, "CustomFlyoutListButtonTemplate")
		btn = parent.ButtonList[idx]
		btn.index = idx
		btn:SetMovable(true)
		btn:RegisterForDrag("LeftButton")
		btn:SetNormalTexture("Interface\\Buttons\\UI-Quickslot")
		btn:SetAttribute("checkselfcast", true)
		btn:SetAttribute("checkfocuscast", true)
		btn:SetAttribute("unit2", "player") --right-click self cast
		btn:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		if (LBFMasterGroup) then
			LBFMasterGroup:AddButton(btn)
		end
	end
	SetListButtonAnchor(btn, parent, idx, expandDir)
	btn:Show()
end

local function SetFlyoutFrameAnchor(FlyoutListFrame, arrowBtn, expandDir)
	FlyoutListFrame:ClearAllPoints()
	if expandDir == DIR_UP then
		FlyoutListFrame:SetPoint("BOTTOM", arrowBtn, "TOP", 0, 5)
	elseif expandDir == DIR_LEFT then
		FlyoutListFrame:SetPoint("RIGHT", arrowBtn, "LEFT", -5, 0)
	elseif expandDir == DIR_DOWN then
		FlyoutListFrame:SetPoint("TOP", arrowBtn, "BOTTOM", 0, -5)
	else --if expandDir == DIR_LEFT then
		arrowBtn.FlyoutListFrame:SetPoint("LEFT", arrowBtn, "RIGHT", 5, 0)
	end
end

local function SetFlyoutFrameSize(flyoutListFrame, expandDir, bcount)
	local width = BUTTON_PLACE_SIZE + 2 * BUTTON_PLACE_OFFSET
	local height = bcount * (BUTTON_PLACE_SIZE + BUTTON_PLACE_OFFSET) + BUTTON_PLACE_OFFSET
	
	if expandDir == DIR_UP or expandDir == DIR_DOWN then
		flyoutListFrame:SetWidth(width)
		flyoutListFrame:SetHeight(height)
	else
		flyoutListFrame:SetWidth(height)
		flyoutListFrame:SetHeight(width)
	end
end

local function GetFlyoutListButtonsCount(listTable)
	local count = #listTable
	local settingsCount = count
	if SettingsMode then
		if count < MIN_VISIBLE_BUTTONS_SETTINGS_MODE then
			settingsCount = MIN_VISIBLE_BUTTONS_SETTINGS_MODE
		else
			settingsCount = count + 1
		end
	end
	return count, settingsCount
end

local function HideListButton(self)
	self.tooltipValue = nil
	self.tooltipType = nil
	self:SetAttribute("type", "none")
	_G[self:GetName().."Icon"]:Hide()
	_G[self:GetName().."Cooldown"]:Hide()
	_G[self:GetName().."Border"]:Hide()
	_G[self:GetName().."Count"]:Hide()
	_G[self:GetName().."Name"]:SetText("")
	self:SetNormalTexture("Interface\\Buttons\\UI-Quickslot")
	self:SetChecked(0)
	self:Hide()
end

function FlyoutButtonCustom_Update(arrowBtn) --update for arrow button and flyout list
	if InCombatLockdown() then
		return
	end
	if not(arrowBtn.parent.customFlyout) or InCombatLockdown() then
		return
	end
	
	local count, settingsCount = GetFlyoutListButtonsCount(arrowBtn.FlyoutListTable)
	
	if not(arrowBtn.FlyoutListFrame) then
		arrowBtn.FlyoutListFrame = CreateFrame("Frame", arrowBtn.parent:GetName().."CustomFlyoutListFrame", arrowBtn, "CustomFlyoutListFrameTemplate")
		arrowBtn:SetFrameRef("FlyoutListFrame_Ref", arrowBtn.FlyoutListFrame)
		arrowBtn.FlyoutListFrame.ButtonList = {}
		SetFlyoutFrameAnchor(arrowBtn.FlyoutListFrame, arrowBtn, arrowBtn:GetAttribute("expandDir"))

		--self, stateid, newstate
		arrowBtn.FlyoutListFrame:SetAttribute("_onstate-combat", FlyoutListFrameCombatSnippet)
		RegisterStateDriver(arrowBtn.FlyoutListFrame, "combat", "[nocombat] 0; [combat] 1;")

		arrowBtn.FlyoutListFrame:SetAttribute("_onstate-vehicle", FlyoutListFrameVehicleSnippet)
		RegisterStateDriver(arrowBtn.FlyoutListFrame, "vehicle", "[novehicleui] 0; [vehicleui] 1;")
	end
	local flf = arrowBtn.FlyoutListFrame
	SetFlyoutFrameSize(flf, arrowBtn:GetAttribute("expandDir"), settingsCount)
	arrowBtn:SetAttribute("bcount", count)
	arrowBtn.FlyoutListFrame:SetAttribute("bcount", count)
	
	--cleanup
	for _, v in ipairs(flf.ButtonList) do
		HideListButton(v)
	end
	
	for i, v in ipairs(arrowBtn.FlyoutListTable) do
		local fltb = arrowBtn.FlyoutListTable[i]
		if SettingsMode or fltb["command"] then
			AttachListButton(flf, i, arrowBtn:GetAttribute("expandDir"))
		
			local command = fltb["command"]
			if ACCEPTABLE_COMMANDS[command] then
				SetListButton(flf.ButtonList[i], fltb["command"], fltb["value"], fltb["subValue"])
			end
		end
	end
	
	--add empty buttons
	if SettingsMode then
		for i = #arrowBtn.FlyoutListTable + 1, settingsCount do
			AttachListButton(flf, i, arrowBtn:GetAttribute("expandDir"))
		end
	end
end

function CustomFlyoutArrowButton_PostClick(self, button)
	if not(InCombatLockdown()) then
		if button == "LeftButton" then
			if SettingsMode then
				FlyoutButtonCustom_Update(self)
			else
				--cleanup
				for i, v in ipairs(self.FlyoutListFrame.ButtonList) do
					if self.FlyoutListTable[i] and self.FlyoutListTable[i]["command"] then
						v:Show()
					else
						HideListButton(v)
					end
				end
				local count = GetFlyoutListButtonsCount(self.FlyoutListTable)
				SetFlyoutFrameSize(self.FlyoutListFrame, self:GetAttribute("expandDir"), count)
			end
		elseif button == "RightButton" then
			local expandDir = self:GetAttribute("expandDir")
			expandDir = expandDir + 1
			if expandDir > DIR_RIGHT then
				expandDir = DIR_UP
			end

			local currentSet = GetActiveTalentGroup()
			SVUIFlyoutCustom_Settings["expandDir"][currentSet][self:GetParent():GetName()] = expandDir

			SetArrowFrameAnchor(self, self:GetParent(), expandDir)
			SetArrowFrameSize(self, expandDir)
			SetArrowTextureRotation(self.FlyoutArrow, expandDir, self:GetAttribute("expanded"))
			self:SetAttribute("expandDir", expandDir)

			SetFlyoutFrameAnchor(self.FlyoutListFrame, self, expandDir)
			FlyoutButtonCustom_Update(self)
		end
	end
end

function CustomFlyoutArrowButton_OnEnter(self)
	if self:GetAttribute("expanded") then
		self:SetAlpha(1)
	else
		self:SetAlpha(1 * AlphaMultiplier)
	end
end

function CustomFlyoutArrowButton_OnLeave(self)
	if self:GetAttribute("expanded") then
		self:SetAlpha(1)
	else
		if (self:GetAttribute("bcount") > 0) then
			self:SetAlpha(0.75 * AlphaMultiplier)
		else
			self:SetAlpha(0.5 * AlphaMultiplier)
		end
	end
	CustomFlyoutArrowButton_PostClick(self, "LeftButton")
end

local function SetCursor(command, value, subValue)
	if not(command) then
		return
	end

	ClearCursor()
	if (command == "spell") then
		if (subValue == "MOUNT" or subValue == "CRITTER") then
			--index, creatureName, creatureSpellID, spellName, icon
			local idx = GetCompanionInfoCustom(subValue, value)
			PickupCompanion(subValue, idx)	--("type", index)
		else
			local slot = FindSpellSlot(value, BOOKTYPE_SPELL)
			if slot then
				PickupSpellBookItem(slot, BOOKTYPE_SPELL)
			end
		end
	elseif (command == "item") then
		PickupItem(value)	--itemID or "itemString" or "itemName" or "itemLink"
	elseif (command == "macro") then
		PickupMacro(value)
	end
end

local function GetCursorValues()
	local command, value, subValue = GetCursorInfo()
	if (command == "spell") then
		value = GetSpellName(value, BOOKTYPE_SPELL)
	elseif (command == "item") then
		--nothing to do
	elseif (command == "macro") then		
		subValue = value -- value is macro index	
		value = GetMacroInfo(value) --name, texture, body
	elseif (command == "companion") then
		local _, _, spellId = GetCompanionInfo(subValue, value)
		value = GetSpellInfo(spellId)
		command = "spell"
	end
	return command, value, subValue
end

function CustomFlyoutListButton_PreClick(self, button)
	if InCombatLockdown() then
		ClearCursor()
		return
	end
	
	StoredCursor = {}
	local command, value, subValue = GetCursorValues()
	--print("PreClick "..tostring(command)..", "..tostring(value)..", "..tostring(subValue))
	if button == "LeftButton" then
		if command and SettingsMode then
			StoredCursor.command, StoredCursor.value, StoredCursor.subValue = command, value, subValue
			self:SetAttribute("type", nil) --to avoid click event (spell cast)
		end
	elseif button == "RightButton" then
		StoredCursor.prevCommand = self:GetAttribute("type")
		self:SetAttribute("type", nil) --to avoid click event (spell cast)
		--todo set to action button
	end
end

function CustomFlyoutListButton_PostClick(self, button)
	if InCombatLockdown() then
		return
	end
	
	if button == "LeftButton" and StoredCursor.command then
		--print("PostClick stored cursor "..tostring(StoredCursor.command)..", "..tostring(StoredCursor.value)..", "..tostring(StoredCursor.subValue))
		local arrowBtn = self:GetParent():GetParent()
		local list = arrowBtn.FlyoutListTable
		local command, value, subValue
		if list[self.index] and list[self.index]["command"] then
			command, value, subValue = list[self.index]["command"], list[self.index]["value"], list[self.index]["subValue"]
		end
		SetCursor(StoredCursor.command, StoredCursor.value, StoredCursor.subValue)
		CustomFlyoutListButton_OnReceiveDrag(self)
		if command then
			SetCursor(command, value, subValue)
		end
	elseif button == "RightButton" then
		local arrowBtn = self:GetParent():GetParent()
		local actBtn = arrowBtn:GetParent()
		local list = arrowBtn.FlyoutListTable
		--print("actbtn "..actBtn:GetName()..", action"..tostring(actBtn.action)..", prevCmd "..tostring(StoredCursor.prevCommand))
		if StoredCursor.prevCommand then
			self:SetAttribute("type", StoredCursor.prevCommand)
		end
		if actBtn.action and actBtn.action <= 120 and list[self.index] and list[self.index]["command"] then
			SetCursor(list[self.index]["command"], list[self.index]["value"], list[self.index]["subValue"])
			--print(tostring(actBtn.action))
			PlaceAction(actBtn.action)
			ActionButton_UpdateState(actBtn)
			ActionButton_UpdateFlash(actBtn)
			ClearCursor()
		elseif tostring(actBtn:GetName()):match("^ButtonForge%d+$") and BFButton and BFButton.OnReceiveDrag then
			SetCursor(list[self.index]["command"], list[self.index]["value"], list[self.index]["subValue"])
			BFButton.OnReceiveDrag(actBtn)
			ClearCursor()
		end
		self:SetChecked(nil)
	end
end

function CustomFlyoutList_OnShow(self)
	self:GetParent():SetAlpha(1)
	if ShowBorders then
		self:SetBackdrop({edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", tile = true, tileSize = 8, edgeSize = 8, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
	else
		self:SetBackdrop({edgeFile = ""})
	end

	local ab = self:GetParent()
	if ab then
		--print("OnShow ab "..ab:GetName()..", "..tostring(ab:GetAttribute("expanded")))
		SetArrowTextureRotation(ab.FlyoutArrow, ab:GetAttribute("expandDir"), ab:GetAttribute("expanded"))
	end
end

function CustomFlyoutList_OnHide(self)
	local ab = self:GetParent()
	if ab then
		--print("OnHide ab "..ab:GetName()..", "..tostring(ab:GetAttribute("expanded")))
		SetArrowTextureRotation(ab.FlyoutArrow, ab:GetAttribute("expandDir"), ab:GetAttribute("expanded"))
	end
end

function CustomFlyoutListButton_OnReceiveDrag(self)
	if InCombatLockdown() then
		ClearCursor()
		return
	end

	local command, value, subValue = GetCursorValues()
	--print("OnReceiveDrag "..tostring(command)..", "..tostring(value)..", "..tostring(subValue))
	local arrowBtn = self:GetParent():GetParent()
	local list = arrowBtn.FlyoutListTable

	ClearCursor()
	if list[self.index] and list[self.index]["command"] then
		SetCursor(list[self.index]["command"], list[self.index]["value"], list[self.index]["subValue"])
	end

	local temp = {}
	temp["command"] = command
	temp["value"] = value
	temp["subValue"] = subValue
	
	if list then
		if list[self.index] then
			list[self.index]["command"] = command
			list[self.index]["value"] = value
			list[self.index]["subValue"] = subValue
		else
			for i = 1, self.index - 1 do
				list[i] = list[i] or {}
			end
			table.insert(list, self.index, temp)
		end
	end
	
	local actnBtnName = arrowBtn.parent:GetName()
	local currentSet = GetActiveTalentGroup()
	if command then
		SVUIFlyoutCustom_Settings[actnBtnName] = SVUIFlyoutCustom_Settings[actnBtnName] or {}
		SVUIFlyoutCustom_Settings[actnBtnName][currentSet] = SVUIFlyoutCustom_Settings[actnBtnName][currentSet] or {}
		SVUIFlyoutCustom_Settings[actnBtnName][currentSet][self.index] = SVUIFlyoutCustom_Settings[actnBtnName][currentSet][self.index] or {}
		
		SVUIFlyoutCustom_Settings[actnBtnName][currentSet][self.index]["command"] = command
		SVUIFlyoutCustom_Settings[actnBtnName][currentSet][self.index]["value"] = value
		SVUIFlyoutCustom_Settings[actnBtnName][currentSet][self.index]["subValue"] = subValue
	end
	FlyoutButtonCustom_Update(arrowBtn)
end

function CustomFlyoutListButton_OnDragStart(self)
	if not(SettingsMode) then
		return
	end
	
	local arrowBtn = self:GetParent():GetParent()
	local list = arrowBtn.FlyoutListTable
	local actnBtnName = arrowBtn.parent:GetName()
	local currentSet = GetActiveTalentGroup()

	if list[self.index] and list[self.index]["command"] then
		SetCursor(list[self.index]["command"], list[self.index]["value"], list[self.index]["subValue"])
	end
	
	if self.index == #list then
		table.remove(list, self.index)
		if SVUIFlyoutCustom_Settings[actnBtnName] and SVUIFlyoutCustom_Settings[actnBtnName][currentSet] and SVUIFlyoutCustom_Settings[actnBtnName][currentSet][self.index] then
			SVUIFlyoutCustom_Settings[actnBtnName][currentSet][self.index] = nil
		end
	else
		list[self.index] = {}
		if SVUIFlyoutCustom_Settings[actnBtnName] and SVUIFlyoutCustom_Settings[actnBtnName][currentSet] and SVUIFlyoutCustom_Settings[actnBtnName][currentSet][self.index] then
			SVUIFlyoutCustom_Settings[actnBtnName][currentSet][self.index] = {}
		end
	end
	
	--clear all upper empty slots
	for i = #list, 1, -1 do
		if list[i] and not(list[i]["command"]) then
			table.remove(list, i)
			if SVUIFlyoutCustom_Settings[actnBtnName] and SVUIFlyoutCustom_Settings[actnBtnName][currentSet] and SVUIFlyoutCustom_Settings[actnBtnName][currentSet][i] then
				SVUIFlyoutCustom_Settings[actnBtnName][currentSet][i] = nil
			end
		else
			break
		end
	end
	
	--empty tables to nil
	if SVUIFlyoutCustom_Settings[actnBtnName] then
		if SVUIFlyoutCustom_Settings[actnBtnName][currentSet] then
			if #SVUIFlyoutCustom_Settings[actnBtnName][currentSet] == 0 then
				SVUIFlyoutCustom_Settings[actnBtnName][currentSet] = nil
			end
		end
		if #SVUIFlyoutCustom_Settings[actnBtnName] == 0 then
			SVUIFlyoutCustom_Settings[actnBtnName] = nil
		end
	end
	FlyoutButtonCustom_Update(arrowBtn)
end

function FlyoutButtonCustom_ButtonFacadeCallback(self, SkinID, Gloss, Backdrop, Group, Button, Colors)
	-- If no group is specified, save the data as the root add-on skin.
	-- This will allow the ButtonFacade GUI to display it correctly.
	if not(Group) then
		SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP]["SkinID"] = SkinID
		SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP]["Gloss"] = Gloss
		SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP]["Backdrop"] = Backdrop
		SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP]["Colors"] = Colors
	else
		SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP][Group]["SkinID"] = SkinID
		SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP][Group]["Gloss"] = Gloss
		SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP][Group]["Backdrop"] = Backdrop
		SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP][Group]["Colors"] = Colors
	end
end



--[[ events stuff ]]

local EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
EventFrame:RegisterEvent("VARIABLES_LOADED")


--event handlers
local function EventEnteringCombat(self, ...)
	SettingsMode = false
end

local function DoUpdates(func)
	for _, arrowBtn in pairs(ArrowButtons) do
		local flf = arrowBtn.FlyoutListFrame
		--if arrowBtn.FlyoutListFrame:IsVisible() then
			for i, v in ipairs(arrowBtn.FlyoutListTable) do
				local fltb = arrowBtn.FlyoutListTable[i]
				if fltb["command"] then
					func(flf.ButtonList[i], fltb["command"], fltb["value"], fltb["subValue"])
				end
			end
		--end
	end
end

local function EventTalentGroupChanged(self, ...)
	if InCombatLockdown() then
		return
	end
	
	HideFlyoutFrames()
	ArrowButtons = {}
	RegisterFrames()
	AttachToAllActionButtons()
	
	DoUpdates(ListButton_SetTooltip)
	DoUpdates(ListButton_UpdateTexture)
	DoUpdates(ListButton_UpdateCooldown)
	DoUpdates(ListButton_UpdateChecked)
	DoUpdates(ListButton_UpdateEquipped)
	DoUpdates(ListButton_UpdateUsable)
	DoUpdates(ListButton_UpdateButtonText)
end

local function EventLearnedSpellInTab(self, ...)
	DoUpdates(ListButton_SetTooltip)
	DoUpdates(ListButton_UpdateTexture)
	DoUpdates(ListButton_UpdateCooldown)
	DoUpdates(ListButton_UpdateChecked)
	DoUpdates(ListButton_UpdateUsable)
	DoUpdates(ListButton_UpdateButtonText)
end

local function EventCooldownUpdate(self, ...)
	DoUpdates(ListButton_UpdateCooldown)
end

local function EventCheckedUpdate(self, ...)
	DoUpdates(ListButton_UpdateChecked)
end

local function EventEquippedUpdate(self, ...)
	DoUpdates(ListButton_UpdateEquipped)
	DoUpdates(ListButton_UpdateButtonText)
	DoUpdates(ListButton_UpdateCooldown)
end

local function EventUsableUpdate(self, ...)
	DoUpdates(ListButton_UpdateUsable)
	DoUpdates(ListButton_UpdateButtonText)
	DoUpdates(ListButton_UpdateCooldown)
end

local function EventMacroUpdate(self, ...)
	if InCombatLockdown() then
		return
	end

	for _, arrowBtn in pairs(ArrowButtons) do
		local flf = arrowBtn.FlyoutListFrame
		for i, v in ipairs(arrowBtn.FlyoutListTable) do
			local fltb = arrowBtn.FlyoutListTable[i]
			if fltb["command"] == "macro" then
				--func(flf.ButtonList[i], fltb["command"], fltb["value"], fltb["subValue"])
				--command = "macro", value = macro name, subValue = macro index
				local command, subValue = fltb["command"], fltb["subValue"]
				local value = GetMacroInfo(subValue)

				local actnBtnName = arrowBtn.parent:GetName()
				local currentSet = GetActiveTalentGroup()
				if value then
					SVUIFlyoutCustom_Settings[actnBtnName] = SVUIFlyoutCustom_Settings[actnBtnName] or {}
					SVUIFlyoutCustom_Settings[actnBtnName][currentSet] = SVUIFlyoutCustom_Settings[actnBtnName][currentSet] or {}
					SVUIFlyoutCustom_Settings[actnBtnName][currentSet][i] = SVUIFlyoutCustom_Settings[actnBtnName][currentSet][i] or {}
					
					SVUIFlyoutCustom_Settings[actnBtnName][currentSet][i]["command"] = command
					SVUIFlyoutCustom_Settings[actnBtnName][currentSet][i]["value"] = value
					SVUIFlyoutCustom_Settings[actnBtnName][currentSet][i]["subValue"] = subValue
				else
					fltb["command"], fltb["value"], fltb["subValue"] = nil, nil, nil
					--empty tables to nil
					if SVUIFlyoutCustom_Settings[actnBtnName] then
						if SVUIFlyoutCustom_Settings[actnBtnName][currentSet] then
							if #SVUIFlyoutCustom_Settings[actnBtnName][currentSet] == 0 then
								SVUIFlyoutCustom_Settings[actnBtnName][currentSet] = nil
							end
						end
						if #SVUIFlyoutCustom_Settings[actnBtnName] == 0 then
							SVUIFlyoutCustom_Settings[actnBtnName] = nil
						end
					end
				end
				FlyoutButtonCustom_Update(arrowBtn)
				--SetListButton(button, command, value, subValue)
			end
		end
	end
end

local function RegisterEvents()
	--event table
	EventHandlersTable = {
		--common
		["PLAYER_REGEN_DISABLED"] 			= EventEnteringCombat,
		["ACTIVE_TALENT_GROUP_CHANGED"] 	= EventTalentGroupChanged,
		["LEARNED_SPELL_IN_TAB"] 			= EventLearnedSpellInTab,
		--cooldown
		["SPELL_UPDATE_COOLDOWN"] 			= EventCooldownUpdate,
		["BAG_UPDATE_COOLDOWN"] 			= EventCooldownUpdate,
		["ACTIONBAR_UPDATE_COOLDOWN"] 		= EventCooldownUpdate,
		["UPDATE_SHAPESHIFT_COOLDOWN"] 		= EventCooldownUpdate,
		--checked
		["TRADE_SKILL_SHOW"] 				= EventCheckedUpdate,
		["TRADE_SKILL_CLOSE"] 				= EventCheckedUpdate,
		["ARCHAEOLOGY_TOGGLE"] 				= EventCheckedUpdate,
		["ARCHAEOLOGY_CLOSED"] 				= EventCheckedUpdate,
		["COMPANION_UPDATE"] 				= EventCheckedUpdate,
		["CURRENT_SPELL_CAST_CHANGED"] 		= EventCheckedUpdate,
		["ACTIONBAR_UPDATE_STATE"] 			= EventCheckedUpdate,
		["PLAYER_ENTER_COMBAT"] 			= EventCheckedUpdate,
		["PLAYER_LEAVE_COMBAT"] 			= EventCheckedUpdate,
		["START_AUTOREPEAT_SPELL"] 			= EventCheckedUpdate,
		["STOP_AUTOREPEAT_SPELL"] 			= EventCheckedUpdate,
		["UPDATE_BONUS_ACTIONBAR"] 			= EventCheckedUpdate,
		["ACTIONBAR_PAGE_CHANGED"] 			= EventCheckedUpdate,
		--equipment
		["PLAYER_EQUIPMENT_CHANGED"] 		= EventEquippedUpdate,
		--usable
		["BAG_UPDATE"] 						= EventUsableUpdate,
		--["UNIT_INVENTORY_CHANGED"] 			= EventUsableUpdate,
		["SPELL_UPDATE_USABLE"] 			= EventUsableUpdate,
		["PLAYER_CONTROL_LOST"] 			= EventUsableUpdate,
		["PLAYER_CONTROL_GAINED"] 			= EventUsableUpdate,
		["UPDATE_BONUS_ACTIONBAR"] 			= EventUsableUpdate,
		["ACTIONBAR_UPDATE_USABLE"] 		= EventUsableUpdate,
		["VEHICLE_UPDATE"] 					= EventUsableUpdate,
		["UPDATE_WORLD_STATES"] 			= EventUsableUpdate,
		--macro
		["UPDATE_MACROS"]					= EventMacroUpdate,
	}
	
	--EventFrame:RegisterEvent("ACTIONBAR_HIDEGRID")
	EventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	EventFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	EventFrame:RegisterEvent("LEARNED_SPELL_IN_TAB")
	--cooldown events
	EventFrame:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	EventFrame:RegisterEvent("BAG_UPDATE_COOLDOWN")
	EventFrame:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
	EventFrame:RegisterEvent("UPDATE_SHAPESHIFT_COOLDOWN")
	--checked events
	EventFrame:RegisterEvent("TRADE_SKILL_SHOW")
	EventFrame:RegisterEvent("TRADE_SKILL_CLOSE")
	EventFrame:RegisterEvent("ARCHAEOLOGY_TOGGLE")
	EventFrame:RegisterEvent("ARCHAEOLOGY_CLOSED")
	EventFrame:RegisterEvent("COMPANION_UPDATE")
	EventFrame:RegisterEvent("CURRENT_SPELL_CAST_CHANGED")
	EventFrame:RegisterEvent("ACTIONBAR_UPDATE_STATE")
	EventFrame:RegisterEvent("PLAYER_ENTER_COMBAT")
	EventFrame:RegisterEvent("PLAYER_LEAVE_COMBAT")
	EventFrame:RegisterEvent("START_AUTOREPEAT_SPELL")
	EventFrame:RegisterEvent("STOP_AUTOREPEAT_SPELL")
	--equipment events
	EventFrame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
	--usable events
	EventFrame:RegisterEvent("BAG_UPDATE")
	--EventFrame:RegisterEvent("UNIT_INVENTORY_CHANGED")
	EventFrame:RegisterEvent("SPELL_UPDATE_USABLE")
	EventFrame:RegisterEvent("PLAYER_CONTROL_LOST")
	EventFrame:RegisterEvent("PLAYER_CONTROL_GAINED")
	EventFrame:RegisterEvent("UPDATE_BONUS_ACTIONBAR")
	EventFrame:RegisterEvent("ACTIONBAR_UPDATE_USABLE")
	EventFrame:RegisterEvent("VEHICLE_UPDATE")
	EventFrame:RegisterEvent("UPDATE_WORLD_STATES")
	--todo macro stuff
	EventFrame:RegisterEvent("UPDATE_MACROS")
end

local function FlyoutButtonCustom_SlashHandler(msg, editbox)
	local command, rest = msg:match("^(%S*)%s*(.-)$")
	-- Any leading non-whitespace is captured into command
	-- the rest (minus leading whitespace) is captured into rest.
	command = string.lower(command)
	
	if (command == "" or command == "help") then
		print("FlyoutButton Custom commands:")
		print("'alpha value' to set arrow button alpha, values 0-1")
		print("'border' to turn on/off button borders")
		print("'mouseover' to turn on/off opening/hiding lists on arrow button mouseover")
		print("'heigth value' to set arrow button height, values 5-36")
		print("'settings' to turn on/off SettingsMode")
		print("'unique' to turn on/off only 1 list visible")
	elseif (command == "settings") then
		if not(InCombatLockdown()) then
			SettingsMode = not(SettingsMode)
			print("FlyoutButtonCustom: SettingsMode = "..tostring(SettingsMode))
			if SettingsMode then
				EventTalentGroupChanged()
				ShowAllFlyoutButtons()
			else
				HideUnusedFlyoutButtons()
			end
		else
			print("FlyoutButtonCustom can't enter SettingsMode while in combat")
		end
	elseif (command == "border") then
		ShowBorders = not(ShowBorders)
		SVUIFlyoutCustom_Settings["ShowBorders"] = ShowBorders
		print("FlyoutButtonCustom: ShowBorders = "..tostring(ShowBorders))

		for _, arrowBtn in pairs(ArrowButtons) do
			local flf = arrowBtn.FlyoutListFrame
			for i, v in ipairs(arrowBtn.FlyoutListTable) do
				local fltb = arrowBtn.FlyoutListTable[i]
				if fltb["command"] then
					ListButton_UpdateTexture(flf.ButtonList[i], fltb["command"], fltb["value"], fltb["subValue"])
				end
			end
		end
	elseif (command == "mouseover") then
		if not(InCombatLockdown()) then
			EnableMouseover = not(EnableMouseover)
			SVUIFlyoutCustom_Settings["EnableMouseover"] = EnableMouseover
			print("FlyoutButtonCustom: EnableMouseover = "..tostring(EnableMouseover))
			if EnableMouseover then
				print("'/reload' required")
			end
			for _, arrowBtn in pairs(ArrowButtons) do
				if EnableMouseover then
					arrowBtn:SetScript("OnEnter", nil)
				else
					arrowBtn:SetScript("OnEnter", CustomFlyoutArrowButton_OnEnter)
				end
			end
		else
			print("FlyoutButtonCustom: can't toggle mouseover while in combat")
		end
	elseif (command == "height") then
		if rest ~= "" then
			local temp = tonumber(rest)
			if not(temp) or (temp < ARROW_BUTTON_MIN_HEIGHT) or (temp > ARROW_BUTTON_MAX_HEIGHT) then
				print("FlyoutButtonCustom: new height value '"..rest.."' out of range "..ARROW_BUTTON_MIN_HEIGHT.."-"..ARROW_BUTTON_MAX_HEIGHT)
			else
				ArrowButtonsHeight = temp
				SVUIFlyoutCustom_Settings["ArrowButtonsHeight"] = ArrowButtonsHeight
				
				for _, arrowBtn in pairs(ArrowButtons) do
					SetArrowFrameSize(arrowBtn, arrowBtn:GetAttribute("expandDir"))
				end
			end
		end
	elseif (command == "alpha") then
		if rest ~= "" then
			local temp = tonumber(rest)
			if not(temp) or (temp < 0) or (temp > 1) then
				print("FlyoutButtonCustom: new alpha value '"..rest.."' out of range 0-1")
			else
				AlphaMultiplier = temp
				SVUIFlyoutCustom_Settings["AlphaMultiplier"] = AlphaMultiplier
				
				for _, arrowBtn in pairs(ArrowButtons) do
					CustomFlyoutArrowButton_OnLeave(arrowBtn)
				end
			end
		end
	elseif (command == "unique") then
		if not(InCombatLockdown()) then
			StubFrame:SetAttribute("unique-list", not(StubFrame:GetAttribute("unique-list")))
			SVUIFlyoutCustom_Settings["UniqueList"] = StubFrame:GetAttribute("unique-list")
			print("FlyoutButtonCustom: unique list = "..tostring(StubFrame:GetAttribute("unique-list")))
		else
			print("FlyoutButtonCustom: can't set while in combat")
		end
	else
		print("FlyoutButtonCustom: unknown command '"..command.."'")
	end
end

local spb_frame = SpellBookFrame
spb_frame:HookScript("OnShow", function()
	if not(InCombatLockdown()) then
		SettingsMode = true
		EventTalentGroupChanged()
		ShowAllFlyoutButtons()
	end
end)
spb_frame:HookScript("OnHide", function()
	SettingsMode = false
	if not(InCombatLockdown()) then
		HideUnusedFlyoutButtons()
	end
end)



local function OnEvent(self, event, ...)
	if EventHandlersTable[event] then
		EventHandlersTable[event](self, ...)
	elseif event == "PLAYER_ENTERING_WORLD" or event == "VARIABLES_LOADED" then
		if EnteringWorldOrVariablesLoaded then
			SVUIFlyoutCustom_Settings = SVUIFlyoutCustom_Settings or {}
			local fbc_s = SVUIFlyoutCustom_Settings
			
			--read settings
			ArrowButtonsHeight = fbc_s["ArrowButtonsHeight"] or ARROW_BUTTON_DEF_HEIGHT
			if fbc_s["ShowBorders"] ~= nil then 
				ShowBorders = fbc_s["ShowBorders"]
			else
				ShowBorders = true
			end
			if fbc_s["EnableMouseover"] ~= nil then 
				EnableMouseover = fbc_s["EnableMouseover"]
			else
				EnableMouseover = false
			end			
			AlphaMultiplier = fbc_s["AlphaMultiplier"] or ARROW_BUTTON_DEF_ALPHA_MULTIPLIER
			if fbc_s["UniqueList"] == nil then
				fbc_s["UniqueList"] = true
			end
			StubFrame:SetAttribute("unique-list", fbc_s["UniqueList"])
			SVUIFlyoutCustom_Settings["expandDir"] = SVUIFlyoutCustom_Settings["expandDir"] or {}
			--for 2 talent sets
			SVUIFlyoutCustom_Settings["expandDir"][1] = SVUIFlyoutCustom_Settings["expandDir"][1] or {}
			SVUIFlyoutCustom_Settings["expandDir"][2] = SVUIFlyoutCustom_Settings["expandDir"][2] or {}
			
			--Button Facade
			if (LibStub) then
				LBF = LibStub("LibButtonFacade", true)
				if (LBF) then
					LBFMasterGroup = LBF:Group("FlyoutButtonCustom")
					LBF:RegisterSkinCallback("FlyoutButtonCustom", FlyoutButtonCustom_ButtonFacadeCallback, BF_Table)
				end
			end
			SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP] = SVUIFlyoutCustom_Settings[BF_SETTINGS_GROUP] or {}
			if (LBFMasterGroup and fbc_s[BF_SETTINGS_GROUP]["SkinID"]) then
				LBFMasterGroup:Skin(fbc_s[BF_SETTINGS_GROUP]["SkinID"], fbc_s[BF_SETTINGS_GROUP]["Gloss"], fbc_s[BF_SETTINGS_GROUP]["Backdrop"], fbc_s[BF_SETTINGS_GROUP]["Colors"])
			end			
			
			RegisterFrames()
			AttachToAllActionButtons()
			RegisterEvents()
			
			SLASH_FBCUSTOM1, SLASH_FBCUSTOM2 = "/fbc", "/fbcustom"
			SlashCmdList["FBCUSTOM"] = FlyoutButtonCustom_SlashHandler
		else
			EnteringWorldOrVariablesLoaded = true
		end
		if event == "PLAYER_ENTERING_WORLD" then EventFrame:UnregisterEvent("PLAYER_ENTERING_WORLD") end
		if event == "VARIABLES_LOADED" then EventFrame:UnregisterEvent("VARIABLES_LOADED") end
	end
end

EventFrame:SetScript("OnEvent", OnEvent)
