------------------------------------------------------------------------
--	LOCALS
------------------------------------------------------------------------
local mk_anim
local colors = SVColors.oUF_colors
----------------------------------------------------------------------------------------------------
-------------[[//||\\------------------CAST BAR UPDATES-----------------------//||\\]]--------------
----------------------------------------------------------------------------------------------------
local CheckInterrupt = function(self, unit)
	if unit == "vehicle" then unit = "player" end

	if self.interrupt and UnitCanAttack("player", unit) then
		self:SetStatusBarColor(0.8,0,0.1)
		if self.Text then
			self.Text:SetTextColor(1,0.2,0.2)
		end
	else
		if unit == "player" then
			self:SetStatusBarColor(0,0.4,0.8)
			if self.Text then
				self.Text:SetTextColor(0.3,0.92,1)
			end
		else
			self:SetStatusBarColor(0.8,0.4,0)
			if self.Text then
				self.Text:SetTextColor(1,1,0)
			end
		end		
	end
end

SVUnit.CheckCast = function(self, unit, name, rank, castid)
	CheckInterrupt(self, unit)
end

SVUnit.CheckChannel = function(self, unit, name, rank)
	CheckInterrupt(self, unit)
end

SVUnit.CustomCastTimeText = function(self, duration)
	self.Time:SetText(("%.1f"):format(self.channeling and duration or self.max - duration))
end

SVUnit.CustomCastDelayText = function(self, duration)
	self.Time:SetText(("%.1f |cffaf5050%s %.1f|r"):format(self.channeling and duration or self.max - duration, self.channeling and "- " or "+", self.delay))
end
----------------------------------------------------------------------------------------------------
-------------[[//||\\-----------------CUSTOM CAST BAR FUNCTIONS---------------//||\\]]--------------
----------------------------------------------------------------------------------------------------
-- Player`s Cast Fader
local PlayerCastbarFade = function(self,event,...)
	local arg1 = ...;
	if arg1 ~= "player" then return end

	if event == "UNIT_SPELLCAST_START" then
		self.fails = nil
		self.isokey = nil
		self.ischanneling = nil
		self:SetAlpha(0)
		self.mask:SetAlpha(1)

		if self.anim:IsPlaying() then
			self.anim:Stop()
		end

	elseif event == "UNIT_SPELLCAST_CHANNEL_START" then
		self:SetAlpha(0)
		self.mask:SetAlpha(1)

		if self.anim:IsPlaying() then
			self.anim:Stop()
		end

		self.iscasting = nil
		self.fails = nil
		self.isokey = nil

	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
		self.fails = nil
		self.isokey = true
		self.fails_a = nil

	elseif (event == "UNIT_SPELLCAST_FAILED" or event == "UNIT_SPELLCAST_FAILED_QUIET") then
		self.fails = true
		self.isokey = nil
		self.fails_a = nil

	elseif (event == "UNIT_SPELLCAST_INTERRUPTED") then
		self.fails = nil
		self.isokey = nil
		self.fails_a = true

	elseif 	event == "UNIT_SPELLCAST_STOP" then
		if self.fails or self.fails_a then
			self:SetBackdropColor(1,0.2,0.2,0.5)
			self.txt:SetText(SPELL_FAILED_FIZZLE)
			self.txt:SetTextColor(1,0,0.4,0.3)
		elseif self.isokey then
			self:SetBackdropColor(0.2,1,0.2,0.5)
			self.txt:SetText(SUCCESS)
			self.txt:SetTextColor(0,1,0.4,0.3)
		end
		
		self.mask:SetAlpha(0)
		self:SetAlpha(0)

		if not self.anim:IsPlaying() then
			self.anim:Play()
		end
	
		elseif event == "UNIT_SPELLCAST_CHANNEL_STOP" then
			self.mask:SetAlpha(0)
			self:SetAlpha(0)

		if self.fails_a then
			self:SetBackdropColor(1,0.2,0.2,0.5)
			self.txt:SetText(SPELL_FAILED_FIZZLE)
			self.txt:SetTextColor(1,0,0.4,0.3)
			if not self.anim:IsPlaying() then
				self.anim:Play()
			end	
		end
	end	
end
SVUnit.PlayerCastbarFade = PlayerCastbarFade
-- Create Player Fade for castbar -----------------------------------------
local set_fadecast = function(self)
	local fade = CreateFrame ("Frame",nil,self)
	fade:SetFrameLevel(2)
	fade:SetPoint("TOPLEFT",self.Castbar,-22,24)
	fade:SetPoint("BOTTOMRIGHT",self.Castbar,0,-18)
	fade:SetBackdrop({bgFile = SVMedia.neonbar})
	fade:SetBackdropColor(0, 0, 0, 0)
	fade:SetAlpha(0)
	fade:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	fade:RegisterEvent("UNIT_SPELLCAST_START")
	fade:RegisterEvent("UNIT_SPELLCAST_STOP")
	fade:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	fade:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	fade:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	fade:RegisterEvent("UNIT_SPELLCAST_FAILED")
	fade:RegisterEvent("UNIT_SPELLCAST_FAILED_QUIET")
	fade.mask = CreateFrame ("Frame",nil,self)
	fade.mask:SetBackdrop({bgFile = SVMedia.neonbar})
	fade.mask:SetPoint("TOPLEFT",self.Castbar,-22,24)
	fade.mask:SetPoint("BOTTOMRIGHT",self.Castbar,0,-18)
	fade.mask:SetFrameLevel(2)
	fade.mask:SetBackdropColor(0, 0, 0, 0)
	fade.mask:SetAlpha(0)
	fade.txt = fade:CreateFontString(nil, "OVERLAY")
	fade.txt:SetFont(SVFonts.damage, 18, "THINOUTLINE")
	fade.txt:SetAllPoints(fade)
	fade.txt:SetJustifyH("CENTER")
	fade.txt:SetJustifyV("CENTER")
	fade.txt:SetText("")
	fade.anim = fade:CreateAnimationGroup("Flash")
	fade.anim.fadein = fade.anim:CreateAnimation("ALPHA", "FadeIn")
	fade.anim.fadein:SetChange(1)
	fade.anim.fadein:SetOrder(1)
	fade.anim.fadeout1 = fade.anim:CreateAnimation("ALPHA", "FadeOut")
	fade.anim.fadeout1:SetChange(-.25)
	fade.anim.fadeout1:SetOrder(2)
	fade.anim.fadeout2 = fade.anim:CreateAnimation("ALPHA", "FadeOut")
	fade.anim.fadeout2:SetChange(-.75)
	fade.anim.fadeout2:SetOrder(3)
	fade.anim.fadein:SetDuration(0)
	fade.anim.fadeout1:SetDuration(.8)
	fade.anim.fadeout2:SetDuration(.4)
	fade:SetScript("OnEvent",SVUnit.PlayerCastbarFade)
end
SVUnit.SetCastFader = set_fadecast
----------------------------------------------------------------------------------------------------
-------------[[//||\\-----------------CAST BAR CONSTRUCTOR--------------------//||\\]]--------------
----------------------------------------------------------------------------------------------------
local create_cast_bar = function(self,unit)
	if unit == "focustarget" or unit == "targettarget" then return end
	
	local cast = CreateFrame("StatusBar", nil, self)
	---------Taken from SVUI:1
	local cbtall = SVMod.Scale(22)
	local cbwidth = SVUnit.unitW
	if unit == "player" then
		cast:SetStatusBarTexture(SVMedia.neonbar)
		cast:SetHeight(cbtall * 1.95)
		cast:SetWidth(cbwidth - SVMod.Scale(22))
		cast:SetPoint("CENTER", self.cbb, "CENTER", SVMod.Scale(11), 0)
		cast:SetFrameLevel(2)
		local cbbg = CreateFrame("Frame", nil, cast)
		SVMod.DeluxePanel(cbbg, cbwidth, (cbtall + 2), "CENTER", cast, "CENTER", SVMod.Scale(-11), 0)
		cbbg:SetBackdropColor(0,0,0,0.2)
		cbbg:SetFrameLevel(0)
		local cbfg = CreateFrame("Frame", nil, cast)
		cbfg:SetAllPoints(cbbg)
		SVMod.SimpleSkin(cbfg)
		cbfg:SetFrameLevel(4)
		SVMod.fixStatBar(cast)
		cast.Spark = cast:CreateTexture(nil, 'OVERLAY')
		cast.Spark:SetHeight(52)
		cast.Spark:SetWidth(25)
		cast.Spark:SetBlendMode('ADD')
		cast.SafeZone = cast:CreateTexture(nil, "ARTWORK")
		cast.SafeZone:SetAllPoints(cast)
		cast.SafeZone:SetTexture(SVMedia.neonbar)
		cast.SafeZone:SetVertexColor(0.1,1,0.2,0.5)
	elseif (unit == "target") then
		cast:SetStatusBarTexture(SVMedia.neonbar)
		cast:SetHeight(cbtall * 2)
		cast:SetWidth(cbwidth - SVMod.Scale(22))
		cast:SetPoint("CENTER", self.cbb, "CENTER", SVMod.Scale(-11), 0)
		cast:SetFrameLevel(2)
		local cbbg = CreateFrame("Frame", nil, cast)
		SVMod.DeluxePanel(cbbg, cbwidth, (cbtall + 2), "CENTER", cast, "CENTER", SVMod.Scale(11), 0)
		cbbg:SetBackdropColor(0,0,0,0.2)
		cbbg:SetFrameLevel(0)
		local cbfg = CreateFrame("Frame", nil, cast)
		cbfg:SetAllPoints(cbbg)
		SVMod.SimpleSkin(cbfg)
		cbfg:SetFrameLevel(4)
		SVMod.fixStatBar(cast)
		cast.Spark = cast:CreateTexture(nil, 'OVERLAY')
		cast.Spark:SetHeight(52)
		cast.Spark:SetWidth(25)
		cast.Spark:SetBlendMode('ADD')
	else
		cast:SetAllPoints(self.cbb)
		cast:SetStatusBarTexture(SVMedia.bar)
		cast:SetBackdrop({bgFile = SVMedia.bar})
		cast:SetBackdropColor(0.1,0.1,0.1,0.7)
		cast:SetStatusBarColor(1,1,0.2)
		SVMod.SimpleSkin(cast)
		cast.Spark = cast:CreateTexture(nil, "OVERLAY")
		cast.Spark:SetWidth(12)
		cast.Spark:SetBlendMode("ADD")
		cast.Spark:SetAlpha(1)
		cast:SetFrameLevel(11)
	end		
	
	if (unit == "player") or (unit == "target") then
		cast.iconbg = CreateFrame("Frame",nil,cast)
		cast.iconbg:SetWidth(SVMod.Scale(22))
		cast.iconbg:SetHeight(SVMod.Scale(22))
		SVMod.SimpleSkin(cast.iconbg)
		if (unit == "player") then
			cast.iconbg:SetPoint("RIGHT",cast,"LEFT",0,0)
		else
			cast.iconbg:SetPoint("LEFT",cast,"RIGHT",0,0)
		end	
		cast.Icon = cast.iconbg:CreateTexture(nil, "BORDER")
		cast.Icon:SetTexCoord(.1,.9,.1,.9)
		cast.Icon:SetGradient("VERTICAL",.5,.5,.5,1,1,1)
		cast.Icon:SetTexCoord(.1,.9,.1,.9)
		cast.Icon:SetPoint("TOPLEFT", cast.iconbg,2,-2)
		cast.Icon:SetPoint("BOTTOMRIGHT",cast.iconbg,-2,2)
	end
	
	cast.Text = cast:CreateFontString(nil, "OVERLAY")
	cast.Time = cast:CreateFontString(nil, "OVERLAY")
	cast.Time:SetShadowOffset(1, -1)
	if unit == "player" then
		cast.Text:SetPoint("LEFT",cast,"LEFT",4,0)
		cast.Text:SetPoint("RIGHT",cast,"RIGHT",-40,0)
		cast.Text:SetFont(SVFonts.font,12)
		cast.Text:SetShadowOffset(1,-1)
		cast.Text:SetTextColor(1,1,1)
		cast.Text:SetJustifyH("LEFT")
		cast.Time:SetPoint("RIGHT",cast,"RIGHT",-4,0)
		cast.Time:SetFont(SVFonts.cooldown, 16, "THINOUTLINE")
		cast.Time:SetTextColor(1,1,1,0.9)
		cast.Time:SetJustifyH("RIGHT")
	elseif unit == "target" then
		cast.Text:SetPoint("RIGHT",cast,"RIGHT",-4,0)
		cast.Text:SetPoint("LEFT",cast,"LEFT",40,0)
		cast.Text:SetFont(SVFonts.font,12)
		cast.Text:SetShadowOffset(1,-1)
		cast.Text:SetTextColor(1,1,1)
		cast.Text:SetJustifyH("RIGHT")
		cast.Time:SetPoint("LEFT",cast,"LEFT",4,0)
		cast.Time:SetFont(SVFonts.cooldown, 16, "THINOUTLINE")
		cast.Time:SetTextColor(1,1,1,0.9)
		cast.Time:SetJustifyH("RIGHT")
	else
		cast.Text:SetPoint("TOPLEFT",cast,"TOPLEFT",40,-4)
		cast.Text:SetPoint("BOTTOMRIGHT",cast,"BOTTOMRIGHT",-4,4)
		cast.Text:SetFont(SVFonts.font,12)
		cast.Text:SetTextColor(1,1,1,0)
		cast.Text:SetJustifyH("RIGHT")
		cast.Time:SetPoint("LEFT",cast,"LEFT",4,0)
		cast.Time:SetFont(SVFonts.pixel, 8, "THINOUTLINE")
		cast.Time:SetTextColor(1,1,1,0.9)
		cast.Time:SetJustifyH("LEFT")
	end
	if(unit == "targettarget") or (unit == "pet") or (unit == "focus") or (unit == "focustarget") then
		cast.Time:Hide()
		cast.Text:Hide()
	end
	if (unit == "player") or (unit == "target") or (unit == "pet") then
		cast.CustomTimeText = SVUnit.CustomCastTimeText
		cast.CustomDelayText = SVUnit.CustomCastDelayText
		cast.PostCastStart = SVUnit.CheckCast
		cast.PostChannelStart = SVUnit.CheckChannel
	end
	
	if unit~="pet" then
		if unit == "target" then
			cast:SetScript("OnShow", function(s)  
				self.Buffs:SetAlpha(0) 
			end)
			cast:SetScript("OnHide", function(s)  
				self.Buffs:SetAlpha(1)
			end)
		end
	end
	
	self.Castbar = cast
end
SVUnit.CreateCast = create_cast_bar