local mk_anim
if SVRoot.myclass == "DEATHKNIGHT" or SVRoot.myclass == "WARRIOR" or SVRoot.myclass == "ROGUE" then 
	mk_anim = false
else
	mk_anim = true
end

----------------------------------------------------------------------------------------------------
-------------[[//||\\------------------RANDOM UNIT FUNCTIONS------------------//||\\]]--------------
----------------------------------------------------------------------------------------------------
SVMod.updateAllElements = function(frame)
	for _, v in ipairs(frame.__elements) do
		v(frame, "UpdateElement", frame.unit)
	end
end
----////flashers///-------
local Flash = function(self, duration)
	if not self.anim then
		SVUnit:SetFlash(self)
	end

	self.anim.fadein:SetDuration(duration)
	self.anim.fadeout:SetDuration(duration)
	self.anim:Play()
end
local StopFlash = function(self)
	if self.anim then
		self.anim:Finish()
	end
end
------------------------------------------------------------------------
--	Combat/Resting Display
------------------------------------------------------------------------
local mk_combat_resting = function(self,pos)
	local pos = pos or "TOPRIGHT"
	local c_ = CreateFrame ("Frame",nil,self.data)
	c_:SetFrameLevel(20)
	cr = c_:CreateFontString(nil, "OVERLAY")
	SVUnit.SetText(cr,SVUnit.supersize,"RIGHT",SVFonts.super,"OUTLINE")
	if self.Portrait then
		cr:SetPoint(pos,self.Portrait,pos,0,0)
	else
		cr:SetPoint(pos,self,pos,0,0)
	end
	c_:RegisterEvent("PLAYER_REGEN_ENABLED")
	c_:RegisterEvent("PLAYER_REGEN_DISABLED")
	c_:RegisterEvent("PLAYER_FLAGS_CHANGED")
	c_:RegisterEvent("PLAYER_ENTERING_WORLD")
	c_:SetScript("OnEvent", function (self,event)
		if InCombatLockdown() or event == "PLAYER_REGEN_DISABLED" then
			cr:SetText("Peril!")
		elseif IsResting() then
			cr:SetText("zzz")
		else
			cr:SetText("")
		end
	end)
end
SVUnit.CombatResting = mk_combat_resting

------------------------------------------------------------------------
--	Animators for Health/Mana
------------------------------------------------------------------------
local SetAnim = function(self,duration,a,b)
	self.anim = self:CreateAnimationGroup("Flash")
	self.anim.fadein = self.anim:CreateAnimation("ALPHA", "FadeIn")
	self.anim.fadein:SetChange(1)
	self.anim.fadein:SetOrder(b)
	self.anim.fadeout = self.anim:CreateAnimation("ALPHA", "FadeOut")
	self.anim.fadeout:SetChange(-1)
	self.anim.fadeout:SetOrder(a)
	self.anim.fadein:SetDuration(duration)
	self.anim.fadeout:SetDuration(duration)
	self.anim:SetLooping("REPEAT")
end
SVUnit.Animated = SetAnim

local mk_low_mana = function(self)
	if not(mk_anim) then return end

		local flashlow = CreateFrame("Frame",nil,self)
		flashlow:SetBackdrop ({bgFile = SVMedia.bar, 
			tile = false })
		flashlow:SetBackdropColor(0.5,0,1,0.3)
		SVMod.SimpleSkin(flashlow,3)
		flashlow:SetAllPoints(self)
		flashlow.left:SetTexture(0.1,0.4,1)
		flashlow.right:SetTexture(0.1,0.4,1)
		flashlow.bottom:SetTexture(0.1,0.4,1)
		flashlow.top:SetTexture(0.1,0.4,1)
		flashlow:SetAlpha(0)
		flashlow:SetFrameLevel(14)
		SVUnit.Animated(flashlow,.5,2,1)
		self.Power.flashlow = flashlow
end
SVUnit.ManaFlash = mk_low_mana
------------------------------------------------------------------------
--	Portrait PostUpdate
------------------------------------------------------------------------
local LegionPortPostUpdate = function(Portrait, unit)
	local englass = select(2, UnitClass(unit))
	if (SVColors.oUF_colors.class[englass]) then
		local r, g, b = unpack(SVColors.oUF_colors.class[englass])
		Portrait:SetBackdropColor(r, g, b)
		Portrait.bg:SetBackdropColor(r, g, b)
	end
	if not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit) then
		Portrait.DEAD:Show()
	else
		Portrait.DEAD:Hide()
	end
end
SVUnit.LegionPictures = LegionPortPostUpdate

local PortraitPostUpdate = function(Portrait, unit)
	local r, g, b
	if not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit) then
		Portrait.DEAD:Show()
	else
		Portrait.DEAD:Hide()
	end
	if not UnitIsPlayer(unit) then
		local t = SVColors.oUF_colors.reaction[UnitReaction(unit, "player")]
		if t ~= nil then
			r, g, b = t[1],t[2],t[3]
		else
			r, g, b = 0.5,0.5,0.5
		end
		if Portrait.bg then
			Portrait.bg:SetBackdropColor(r, g, b)
		end
	else
		local englass = select(2, UnitClass(unit)) 
		r, g, b = unpack(SVColors.oUF_colors.class[englass])
		if Portrait.bg then
			Portrait.bg:SetBackdropColor(r, g, b)
		end
	end
	if unit ~= "target" then
		Portrait:SetCamera(0)
	end
end
SVUnit.PicturePages = PortraitPostUpdate
------------------------------------------------------------------------
--	Check Pet Model Function
------------------------------------------------------------------------
local mk_petmodel = function(self)
	local n_ = 1
	local a = CreateFrame("Frame")
	local b = CreateFrame("Frame")
	a:SetScript("OnEvent",function(self,event,...)
		local arg1 = ...;
		if arg1 ~= "player" then return end
		if not(self:IsShown()) then return end
		b:Show()
	end)
	a:RegisterEvent("UNIT_EXITED_VEHICLE")
	a:RegisterEvent("UNIT_ENTERED_VEHICLE")
	b:SetScript("OnUpdate",function(s,t)
		n_ = n_ -t
		if n_>0 then return end
			n_ = 1
			s:Hide()
			if not(self:IsShown()) then return end
			self.Portrait:SetUnit("pet")
			self.Portrait:SetCamera(0)
		end)
end
SVUnit.CheckPetModel = mk_petmodel
------------------------------------------------------------------------
--	Phasing Handler
------------------------------------------------------------------------
local Phasing = function(self, event)
	local inPhase = UnitInPhase(self.unit)
	local picon = self.PhaseIcon
	picon:SetAlpha(0.5)
	if not UnitIsPlayer(self.unit) then picon:Hide() return end

	if(inPhase) then
		picon:Hide()
		if (self:GetName():match("oUF_VillainParty")) or (self:GetName():match("oUF_VillainGrid")) or (self:GetName():match("oUF_VillainRaid")) then self.Health.value:SetAlpha(1) end
	else
		picon:Show()
		if (self:GetName():match("oUF_VillainParty")) or (self:GetName():match("oUF_VillainGrid")) or (self:GetName():match("oUF_VillainRaid")) then self.Health.value:SetAlpha(0) end
	end
end
SVUnit.Phasing = Phasing
------------------------------------------------------------------------
--	Makin Duh Menus
------------------------------------------------------------------------
function SVUnit.SpawnMenu(self)
	local unit = self.unit:gsub("(.)", string.upper, 1)
	if unit == "Targettarget" then return end
	if _G[unit.."FrameDropDown"] then
		ToggleDropDownMenu(1, nil, _G[unit.."FrameDropDown"], "cursor")
	elseif (self.unit:match("party")) then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor")
	else
		FriendsDropDown.unit = self.unit
		FriendsDropDown.id = self.id
		FriendsDropDown.initialize = RaidFrameDropDown_Initialize
		ToggleDropDownMenu(1, nil, FriendsDropDown, "cursor")
	end
end

------------------------------------------------------------------------
--	Just a command to test buffs/debuffs alignment FROM TUKUI
------------------------------------------------------------------------

local testui = TestUI or function() end
TestUI = function()
	testui()
	UnitAura = function()
		-- name, rank, texture, count, dtype, duration, timeLeft, caster
		return 'penancelol', 'Rank 2', 'Interface\\Icons\\Spell_Holy_Penance', random(5), 'Magic', 0, 0, "player"
	end
	if(oUF) then
		for i, v in pairs(oUF.units) do
			if(v.UNIT_AURA) then
				v:UNIT_AURA("UNIT_AURA", v.unit)
			end
		end
	end
end
SlashCmdList.TestUI = TestUI
SLASH_TestUI1 = "/testui"