SVWatch = {}
SVWatch.IsTrackingCompletedQuests = function()
	return bit.band(WATCHFRAME_FILTER_TYPE, WATCHFRAME_FILTER_COMPLETED_QUESTS) == WATCHFRAME_FILTER_COMPLETED_QUESTS or false;
end
SVWatch.IsTrackingRemoteQuests = function()
	return ( bit.band(WATCHFRAME_FILTER_TYPE, WATCHFRAME_FILTER_REMOTE_ZONES) == WATCHFRAME_FILTER_REMOTE_ZONES ) or false;
end
SVWatch.IsTrackingAchievements = function()
	return ( bit.band(WATCHFRAME_FILTER_TYPE, WATCHFRAME_FILTER_ACHIEVEMENTS) == WATCHFRAME_FILTER_ACHIEVEMENTS ) or false;
end
SVWatch.IsUsingFilters = function()
	if ( WATCHFRAME_FILTER_COMPLETED_QUESTS == nil ) then return false; end
	return true;
end
--------------------------------------
SVWatch.CheckTimers = function()
	local lstTimers = GetQuestTimers();
	local numTimers = 0;
	local filterOK = false;
	if ( lstTimers ) then
		filterOK = true;
		numTimers = 1;
	else
		numTimers = 0;
	end
	return filterOK,numTimers;
end
--------------------------------------
SVWatch.CheckQuests = function()
	local numQuestWatches = GetNumQuestWatches();
	local playerMoney = GetMoney();
	local numQuests = QuestMapUpdateAllQuests();
	local currentMapZone = GetCurrentMapZone();
	local questIndex = 0;
	local numCurrentMapQuests = 0;
	local numLocalQuests = 0;
	local numToShow = 0;
	local filterOK = false;
	local usingFilters = SVWatch.IsUsingFilters();
	if ( not usingFilters ) then
		filterOK = true;
		return filterOK, numQuestWatches or 0;
	elseif ( numQuestWatches == 0 ) then
		return filterOK, numQuestWatches or 0;
	end
	local trackingCompleted = SVWatch.IsTrackingCompletedQuests();
	local trackingRemote = SVWatch.IsTrackingRemoteQuests();
	local localQuests = {};
	table.wipe(localQuests);
	local currentMapQuests = {};
	table.wipe(currentMapQuests);
	localQuests["zone"] = currentMapZone;	
	for i = 1, numQuests do
		local questId = QuestPOIGetQuestIDByVisibleIndex(i);
		currentMapQuests[questId] = i;
		numCurrentMapQuests = numCurrentMapQuests + 1;
	end
	for id in pairs(currentMapQuests) do	
		localQuests[id] = true;
		numLocalQuests = numLocalQuests + 1;
	end	
	for i = 1, numQuestWatches do
		questIndex = GetQuestIndexForWatch(i);
		if ( questIndex ) then
			local title, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(questIndex);
			local requiredMoney = GetQuestLogRequiredMoney(questIndex);			
			local numObjectives = GetNumQuestLeaderBoards(questIndex);
			if ( isComplete and isComplete < 0 ) then
				isComplete = false;
			elseif ( numObjectives == 0 and playerMoney >= requiredMoney ) then
				isComplete = true;		
			end			
			local isLocal = localQuests[questID];
			filterOK = true;
			if ( isComplete and not trackingCompleted ) then 
				filterOK = false;
			elseif ( not isLocal and not trackingRemote ) then
				filterOK = false;
			end
			if ( filterOK ) then
				numToShow = numToShow + 1;
			end
		end
	end
	filterOK = false;
	if ( numToShow > 0 ) then
		filterOK = true;
	end
	return filterOK, numQuestWatches or 0;
end
--------------------------------------------------
SVWatch.CheckAchievements = function()
	local numAchievementWatches = GetNumTrackedAchievements();
	local lstAchievementWatches = GetTrackedAchievements();
	local filterOK = false;	
	local usingFilters = SVWatch.IsUsingFilters();	
	if ( not usingFilters ) then
		filterOK = true;
		return filterOK, numAchievementWatches or 0;
	end		
	local trackingAchievements = SVWatch.IsTrackingAchievements();
	filterOK = false;
	if ( numAchievementWatches > 0 and trackingAchievements ) then
		filterOK = true;
	end	
	return filterOK, numAchievementWatches or 0;
end
------------------------------------------
SVWatch.CheckFilters = function(self)
	self = WatchFrame;
	local filterOK = true;
	local showQuests,numQuests = SVWatch.CheckQuests();
	local showAchievements,numAchievements = SVWatch.CheckAchievements();
	local showTimers, numTimers = SVWatch.CheckTimers();
	filterOK = showQuests or showAchievements or showTimers;				
	local numTracked = numQuests + numAchievements + numTimers;
	WatchFrameTitle:SetText(OBJECTIVES_TRACKER_LABEL.." ("..numTracked..")");
	return filterOK, numQuests or 0, numAchievements or 0, numTimers or 0;
end
--------------------------------------
SVWatch.CheckAutoShow = function(self)
	if not self then return; end
	if ( self:IsShown() ) then
		if ( WatchFrameHeader ) then
			WatchFrameHeader:Show()
		end
		WatchFrameTitle:Show();
		WatchFrameCollapseExpandButton:Show();
		WatchFrameLines:Show();
	end
end
--------------------------------------
SVWatch.CheckAutoHide = function(self)	
	local filterOK, numQuests, numAchievements, numTimers = SVWatch.CheckFilters(WatchFrame);	
	local numTracked = numQuests + numAchievements + numTimers;
	local autoHide = true;
	if ( not self ) then return autoHide,nil; end
	if ( autoHide ) then
		if ( numTracked == 0 ) then
			self:Hide();
		else
			self:Show();
		end
	else
		self:Show();
	end
	SVWatch.CheckAutoShow(self);
	return autoHide or false, self:IsShown() or false;
end
------------------------------------------
SVWatch.CollapseExpand = function(self)		
	SVWatch.Collapsed = WatchFrame.collapsed or false;
	if ( WatchFrameHeader == nil ) then WatchFrame.userCollapsed = true; end
	SVWatch.UserCollapsed = WatchFrame.userCollapsed or false;
end
------------------------------------------
SVWatch.RestoreWatchFrame = function()
	if ( SVWatch.Collapsed ) then
		WatchFrame_Collapse(WatchFrame); 
		WatchFrame.userCollapsed = SVWatch.UserCollapsed;
	else
		WatchFrame_Expand(WatchFrame);
	end
end
------------------------------------------
SVWatch.UpdateWatchFrame = function(self,parent)
	if ( not self ) then self = WatchFrame; end
	if ( self ) then
		WATCHFRAME_MAXLINEWIDTH = self:GetWidth();
		if ( parent ) then
			WATCHFRAME_MAXLINEWIDTH = parent:GetWidth();
		end
	end
end
------------------------------------------
SVWatch.ResetWatchFrame = function(parent)
	local f = WatchFrame;
	f:ClearAllPoints();
	f:SetClampedToScreen( false );
	f:SetParent(parent);
	f:SetWidth( parent:GetWidth()-40);
	f:SetHeight(600);
	f:SetPoint( "TOPLEFT", 40, 0 );
	f.ClearAllPoints = function() end;
	f.SetPoint = function() end;
	f.SetAllPoints = function() end;
	SVWatch.UpdateWatchFrame(f);
end